@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2003 by the Free Software Foundation, Inc.
+# Copyright (C) 2001-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,7 +12,8 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+# USA.
 
 """Confirm a pending action via URL."""
 
@@ -33,6 +34,12 @@ from Mailman.Logging.Syslog import syslog
 _ = i18n._
 i18n.set_language(mm_cfg.DEFAULT_SERVER_LANGUAGE)
 
+try:
+    True, False
+except NameError:
+    True = 1
+    False = 0
+
 
 
 def main():
@@ -54,8 +61,10 @@ def main():
         safelistname = Utils.websafe(listname)
         bad_confirmation(doc, _('No such list <em>%(safelistname)s</em>'))
         doc.AddItem(MailmanLogo())
+        # Send this with a 404 status.
+        print 'Status: 404 Not Found'
         print doc.Format()
-        syslog('error', 'No such list "%s": %s', listname, e)
+        syslog('error', 'confirm: No such list "%s": %s', listname, e)
         return
 
     # Set the language for the list
@@ -64,7 +73,17 @@ def main():
 
     # Get the form data to see if this is a second-step confirmation
     cgidata = cgi.FieldStorage(keep_blank_values=1)
-    cookie = cgidata.getvalue('cookie')
+    try:
+        cookie = cgidata.getfirst('cookie')
+    except TypeError:
+        # Someone crafted a POST with a bad Content-Type:.
+        doc.AddItem(Header(2, _("Error")))
+        doc.AddItem(Bold(_('Invalid options to CGI script.')))
+        # Send this with a 400 status.
+        print 'Status: 400 Bad Request'
+        print doc.Format()
+        return
+
     if cookie == '':
         ask_for_cookie(mlist, doc, _('Confirmation string was empty.'))
         return
@@ -90,12 +109,13 @@ def main():
     %(safecookie)s.
 
     <p>Note that confirmation strings expire approximately
-    %(days)s days after the initial subscription request.  If your
-    confirmation has expired, please try to re-submit your subscription.
+    %(days)s days after the initial request.  They also expire if the
+    request has already been handled in some way.  If your confirmation
+    has expired, please try to re-submit your request.
     Otherwise, <a href="%(confirmurl)s">re-enter</a> your confirmation
     string.''')
 
-    content = Pending.confirm(cookie, expunge=0)
+    content = mlist.pend_confirm(cookie, expunge=False)
     if content is None:
         bad_confirmation(doc, badconfirmstr)
         doc.AddItem(mlist.GetMailmanFooter())
@@ -104,17 +124,17 @@ def main():
 
     try:
         if content[0] == Pending.SUBSCRIPTION:
-            if cgidata.getvalue('cancel'):
+            if cgidata.getfirst('cancel'):
                 subscription_cancel(mlist, doc, cookie)
-            elif cgidata.getvalue('submit'):
+            elif cgidata.getfirst('submit'):
                 subscription_confirm(mlist, doc, cookie, cgidata)
             else:
                 subscription_prompt(mlist, doc, cookie, content[1])
         elif content[0] == Pending.UNSUBSCRIPTION:
             try:
-                if cgidata.getvalue('cancel'):
+                if cgidata.getfirst('cancel'):
                     unsubscription_cancel(mlist, doc, cookie)
-                elif cgidata.getvalue('submit'):
+                elif cgidata.getfirst('submit'):
                     unsubscription_confirm(mlist, doc, cookie)
                 else:
                     unsubscription_prompt(mlist, doc, cookie, *content[1:])
@@ -122,12 +142,12 @@ def main():
                 doc.addError(_("""The address requesting unsubscription is not
                 a member of the mailing list.  Perhaps you have already been
                 unsubscribed, e.g. by the list administrator?"""))
-                # And get rid of this confirmation cookie
-                Pending.confirm(cookie)
+                # Expunge this record from the pending database.
+                expunge(mlist, cookie)
         elif content[0] == Pending.CHANGE_OF_ADDRESS:
-            if cgidata.getvalue('cancel'):
+            if cgidata.getfirst('cancel'):
                 addrchange_cancel(mlist, doc, cookie)
-            elif cgidata.getvalue('submit'):
+            elif cgidata.getfirst('submit'):
                 addrchange_confirm(mlist, doc, cookie)
             else:
                 # Watch out for users who have unsubscribed themselves in the
@@ -138,18 +158,19 @@ def main():
                     doc.addError(_("""The address requesting to be changed has
                     been subsequently unsubscribed.  This request has been
                     cancelled."""))
-                    Pending.confirm(cookie, expunge=1)
+                    # Expunge this record from the pending database.
+                    expunge(mlist, cookie)
         elif content[0] == Pending.HELD_MESSAGE:
-            if cgidata.getvalue('cancel'):
+            if cgidata.getfirst('cancel'):
                 heldmsg_cancel(mlist, doc, cookie)
-            elif cgidata.getvalue('submit'):
+            elif cgidata.getfirst('submit'):
                 heldmsg_confirm(mlist, doc, cookie)
             else:
                 heldmsg_prompt(mlist, doc, cookie, *content[1:])
         elif content[0] == Pending.RE_ENABLE:
-            if cgidata.getvalue('cancel'):
+            if cgidata.getfirst('cancel'):
                 reenable_cancel(mlist, doc, cookie)
-            elif cgidata.getvalue('submit'):
+            elif cgidata.getfirst('submit'):
                 reenable_confirm(mlist, doc, cookie)
             else:
                 reenable_prompt(mlist, doc, cookie, *content[1:])
@@ -170,6 +191,17 @@ def bad_confirmation(doc, extra=''):
     doc.AddItem(extra)
 
 
+def expunge(mlist, cookie):
+    # Expunge this record from the list's pending database.  This requires
+    # that the list lock be acquired, however the list doesn't need to be
+    # saved because this operation doesn't touch the config.pck file.
+    mlist.Lock()
+    try:
+        mlist.pend_confirm(cookie, expunge=True)
+    finally:
+        mlist.Unlock()
+
+
 
 def ask_for_cookie(mlist, doc, extra=''):
     title = _('Enter confirmation cookie')
@@ -235,9 +267,10 @@ def subscription_prompt(mlist, doc, cookie, userdesc):
     <p>Note: your password will be emailed to you once your subscription is
     confirmed.  You can change it by visiting your personal options page.
 
-    <p>Or hit <em>Cancel and discard</em> to cancel this subscription
-    request.""") + '<p><hr>'
-    if mlist.subscribe_policy in (2, 3):
+    <p>Or hit <em>Cancel my subscription request</em> if you no longer want to
+    subscribe to this list.""") + '<p><hr>'
+    if (mlist.subscribe_policy in (2, 3) and
+            not getattr(userdesc, 'invitation', False)):
         # Confirmation is required
         result = _("""Your confirmation is required in order to continue with
         the subscription request to the mailing list <em>%(listname)s</em>.
@@ -281,8 +314,8 @@ def subscription_prompt(mlist, doc, cookie, userdesc):
     table.AddRow([Hidden('cookie', cookie)])
     table.AddCellInfo(table.GetCurrentRowIndex(), 0, colspan=2)
     table.AddRow([
-        Label(SubmitButton('cancel', _('Cancel my subscription request'))),
-        SubmitButton('submit', _('Subscribe to list %(listname)s'))
+        Label(SubmitButton('submit', _('Subscribe to list %(listname)s'))),
+        SubmitButton('cancel', _('Cancel my subscription request'))
         ])
     form.AddItem(table)
     doc.AddItem(form)
@@ -290,8 +323,12 @@ def subscription_prompt(mlist, doc, cookie, userdesc):
 
 
 def subscription_cancel(mlist, doc, cookie):
-    # Discard this cookie
-    userdesc = Pending.confirm(cookie, expunge=1)[1]
+    mlist.Lock()
+    try:
+        # Discard this cookie
+        userdesc = mlist.pend_confirm(cookie)[1]
+    finally:
+        mlist.Unlock()
     lang = userdesc.language
     i18n.set_language(lang)
     doc.set_language(lang)
@@ -312,20 +349,20 @@ def subscription_confirm(mlist, doc, cookie, cgidata):
         try:
             # Some pending values may be overridden in the form.  email of
             # course is hardcoded. ;)
-            lang = cgidata.getvalue('language')
+            lang = cgidata.getfirst('language')
             if not Utils.IsLanguage(lang):
                 lang = mlist.preferred_language
             i18n.set_language(lang)
             doc.set_language(lang)
             if cgidata.has_key('digests'):
                 try:
-                    digest = int(cgidata.getvalue('digests'))
+                    digest = int(cgidata.getfirst('digests'))
                 except ValueError:
                     digest = None
             else:
                 digest = None
-            userdesc = Pending.confirm(cookie, expunge=0)[1]
-            fullname = cgidata.getvalue('realname', None)
+            userdesc = mlist.pend_confirm(cookie, expunge=False)[1]
+            fullname = cgidata.getfirst('realname', None)
             if fullname is not None:
                 fullname = Utils.canonstr(fullname, lang)
             overrides = UserDesc(fullname=fullname, digest=digest, lang=lang)
@@ -348,6 +385,11 @@ def subscription_confirm(mlist, doc, cookie, cgidata):
             address that has already been unsubscribed.'''))
         except Errors.MMAlreadyAMember:
             doc.addError(_("You are already a member of this mailing list!"))
+        except Errors.MembershipIsBanned:
+            owneraddr = mlist.GetOwnerEmail()
+            doc.addError(_("""You are currently banned from subscribing to
+            this list.  If you think this restriction is erroneous, please
+            contact the list owners at %(owneraddr)s."""))
         except Errors.HostileSubscriptionError:
             doc.addError(_("""\
             You were not invited to this mailing list.  The invitation has
@@ -379,8 +421,8 @@ def subscription_confirm(mlist, doc, cookie, cgidata):
 
 
 def unsubscription_cancel(mlist, doc, cookie):
-    # Discard this cookie
-    Pending.confirm(cookie, expunge=1)
+    # Expunge this record from the pending database
+    expunge(mlist, cookie)
     doc.AddItem(_('You have canceled your unsubscription request.'))
 
 
@@ -397,7 +439,7 @@ def unsubscription_confirm(mlist, doc, cookie):
         try:
             # Do this in two steps so we can get the preferred language for
             # the user who is unsubscribing.
-            op, addr = Pending.confirm(cookie, expunge=0)
+            op, addr = mlist.pend_confirm(cookie, expunge=False)
             lang = mlist.getMemberLanguage(addr)
             i18n.set_language(lang)
             doc.set_language(lang)
@@ -441,7 +483,7 @@ def unsubscription_prompt(mlist, doc, cookie, addr):
     if fullname is None:
         fullname = _('<em>Not available</em>')
     else:
-        fullname = Utils.uncanonstr(fullname, lang)
+        fullname = Utils.websafe(Utils.uncanonstr(fullname, lang))
     table.AddRow([_("""Your confirmation is required in order to complete the
     unsubscription request from the mailing list <em>%(listname)s</em>.  You
     are currently subscribed with
@@ -467,8 +509,8 @@ def unsubscription_prompt(mlist, doc, cookie, addr):
 
 
 def addrchange_cancel(mlist, doc, cookie):
-    # Discard this cookie
-    Pending.confirm(cookie, expunge=1)
+    # Expunge this record from the pending database
+    expunge(mlist, cookie)
     doc.AddItem(_('You have canceled your change of address request.'))
 
 
@@ -485,7 +527,8 @@ def addrchange_confirm(mlist, doc, cookie):
         try:
             # Do this in two steps so we can get the preferred language for
             # the user who is unsubscribing.
-            op, oldaddr, newaddr, globally = Pending.confirm(cookie, expunge=0)
+            op, oldaddr, newaddr, globally = mlist.pend_confirm(
+                cookie, expunge=False)
             lang = mlist.getMemberLanguage(oldaddr)
             i18n.set_language(lang)
             doc.set_language(lang)
@@ -494,6 +537,18 @@ def addrchange_confirm(mlist, doc, cookie):
             bad_confirmation(doc, _('''Invalid confirmation string.  It is
             possible that you are attempting to confirm a request for an
             address that has already been unsubscribed.'''))
+        except Errors.MembershipIsBanned:
+            owneraddr = mlist.GetOwnerEmail()
+            realname = mlist.real_name
+            doc.addError(_("""%(newaddr)s is banned from subscribing to the
+            %(realname)s list.  If you think this restriction is erroneous,
+            please contact the list owners at %(owneraddr)s."""))
+        except Errors.MMAlreadyAMember:
+            realname = mlist.real_name
+            bad_confirmation(doc, _("""%(newaddr)s is already a member of
+            the %(realname)s list.  It is possible that you are attempting
+            to confirm a request for an address that has already been
+            subscribed."""))
         else:
             # The response
             listname = mlist.real_name
@@ -530,7 +585,7 @@ def addrchange_prompt(mlist, doc, cookie, oldaddr, newaddr, globally):
     if fullname is None:
         fullname = _('<em>Not available</em>')
     else:
-        fullname = Utils.uncanonstr(fullname, lang)
+        fullname = Utils.websafe(Utils.uncanonstr(fullname, lang))
     if globally:
         globallys = _('globally')
     else:
@@ -565,14 +620,14 @@ def addrchange_prompt(mlist, doc, cookie, oldaddr, newaddr, globally):
 
 
 def heldmsg_cancel(mlist, doc, cookie):
-    # Discard this cookie
     title = _('Continue awaiting approval')
     doc.SetTitle(title)
     table = Table(border=0, width='100%')
     table.AddRow([Center(Bold(FontAttr(title, size='+1')))])
     table.AddCellInfo(table.GetCurrentRowIndex(), 0,
                       bgcolor=mm_cfg.WEB_HEADER_COLOR)
-    Pending.confirm(cookie, expunge=1)
+    # Expunge this record from the pending database.
+    expunge(mlist, cookie)
     table.AddRow([_('''Okay, the list moderator will still have the
     opportunity to approve or reject this message.''')])
     doc.AddItem(table)
@@ -591,16 +646,18 @@ def heldmsg_confirm(mlist, doc, cookie):
         try:
             # Do this in two steps so we can get the preferred language for
             # the user who posted the message.
-            op, id = Pending.confirm(cookie, expunge=1)
+            subject = 'n/a'
+            op, id = mlist.pend_confirm(cookie)
             ign, sender, msgsubject, ign, ign, ign = mlist.GetRecord(id)
-            subject = Utils.websafe(msgsubject)
             lang = mlist.getMemberLanguage(sender)
+            subject = Utils.websafe(Utils.oneline(msgsubject,
+                                                  Utils.GetCharSet(lang)))
             i18n.set_language(lang)
             doc.set_language(lang)
             # Discard the message
             mlist.HandleRequest(id, mm_cfg.DISCARD,
                                 _('Sender discarded message via web.'))
-        except Errors.LostHeldMessage:
+        except (Errors.LostHeldMessage, KeyError):
             bad_confirmation(doc, _('''The held message with the Subject:
             header <em>%(subject)s</em> could not be found.  The most likely
             reason for this is that the list moderator has already approved or
@@ -659,7 +716,7 @@ def heldmsg_prompt(mlist, doc, cookie, id):
     i18n.set_language(lang)
     doc.set_language(lang)
 
-    subject = Utils.websafe(msgsubject)
+    subject = Utils.websafe(Utils.oneline(msgsubject, Utils.GetCharSet(lang)))
     reason = Utils.websafe(_(givenreason))
     listname = mlist.real_name
     table.AddRow([_('''Your confirmation is required in order to cancel the
@@ -708,7 +765,7 @@ def reenable_confirm(mlist, doc, cookie):
         try:
             # Do this in two steps so we can get the preferred language for
             # the user who is unsubscribing.
-            op, listname, addr = Pending.confirm(cookie, expunge=0)
+            op, listname, addr = mlist.pend_confirm(cookie, expunge=False)
             lang = mlist.getMemberLanguage(addr)
             i18n.set_language(lang)
             doc.set_language(lang)
@@ -758,7 +815,8 @@ def reenable_prompt(mlist, doc, cookie, list, member):
         <a href="%(listinfourl)s">list information page</a>.""")])
         return
 
-    date = time.strftime('%A, %B %d, %Y', info.date + (0,) * 6)
+    date = time.strftime('%A, %B %d, %Y',
+                         time.localtime(time.mktime(info.date + (0,)*6)))
     daysleft = int(info.noticesleft *
                    mlist.bounce_you_are_disabled_warnings_interval /
                    mm_cfg.days(1))
@@ -769,7 +827,7 @@ def reenable_prompt(mlist, doc, cookie, list, member):
     if username is None:
         username = _('<em>not available</em>')
     else:
-        username = Utils.uncanonstr(username, lang)
+        username = Utils.websafe(Utils.uncanonstr(username, lang))
 
     table.AddRow([_("""Your membership in the %(realname)s mailing list is
     currently disabled due to excessive bounces.  Your confirmation is
