@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2016 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -119,7 +119,7 @@ def main():
 
     cgidata = cgi.FieldStorage()
     try:
-        username = cgidata.getvalue('username', '')
+        username = cgidata.getfirst('username', '')
     except TypeError:
         # Someone crafted a POST with a bad Content-Type:.
         doc.AddItem(Header(2, _("Error")))
@@ -128,7 +128,7 @@ def main():
         print 'Status: 400 Bad Request'
         print doc.Format()
         return
-    password = cgidata.getvalue('password', '')
+    password = cgidata.getfirst('password', '')
 
     is_auth = 0
     realname = mlist.real_name
@@ -142,6 +142,13 @@ def main():
         if cgidata.has_key('submit'):
             # This is a re-authorization attempt
             message = Bold(FontSize('+1', _('Authorization failed.'))).Format()
+            remote = os.environ.get('HTTP_FORWARDED_FOR',
+                     os.environ.get('HTTP_X_FORWARDED_FOR',
+                     os.environ.get('REMOTE_ADDR',
+                                    'unidentified origin')))
+            syslog('security',
+                 'Authorization failed (private): user=%s: list=%s: remote=%s',
+                   username, listname, remote)
             # give an HTTP 401 for authentication failure
             print 'Status: 401 Unauthorized'
         # Are we processing a password reminder from the login screen?
